
#include <avr/io.h>
#include <stdbool.h>
#include "leddrv.h"

#define FADER_CNT 8

typedef struct  
{
    uint8_t start;
    uint8_t end;
    uint8_t duration;
    uint8_t currtime;
    bool running;
}   faderchannel_t;


faderchannel_t fader_ch[FADER_CNT];

void fader_init()
{
    for(uint8_t i = 0; i < FADER_CNT; i++)
    {
        fader_ch[i].duration = 0;
        fader_ch[i].currtime = 0;
    }
}

void fader_tick()
{
    int16_t brightness;
    for(uint8_t i = 0; i < FADER_CNT; i++)
    {
        if(fader_ch[i].running == false)
            continue;

        if(fader_ch[i].currtime == fader_ch[i].duration)
        {
            leddrv_set(i, fader_ch[i].end);
            fader_ch[i].running = false;
            continue;
        }

        int16_t diff = fader_ch[i].end - fader_ch[i].start;
        brightness = fader_ch[i].start + (diff * fader_ch[i].currtime) / fader_ch[i].duration;
        
        leddrv_set(i, brightness);
        
        fader_ch[i].currtime++;
    }
}

void fader_set(uint8_t led, uint8_t brightness, uint8_t duration)
{
    fader_ch[led].start = leddrv_get(led);

    fader_ch[led].end = brightness;
    fader_ch[led].duration = duration;
    fader_ch[led].currtime = 0;
    fader_ch[led].running = true;
}

bool fader_running(uint8_t led)
{
    return fader_ch[led].running;
}