/*
 * leddrv.c
 *
 * Created: 28.04.2020 21:28:06
 *  Author: Chris
 */ 

#include <avr/io.h>
#include "leddrv.h"


uint8_t leddrv_brightness[LEDDRV_CHANNELS];


uint8_t leddrv_states[LEDDRV_STATES_CNT];

uint8_t leddrv_times[LEDDRV_STATES_CNT] = {
    1,2,4,8,16,32,64,128
};

uint8_t leddrv_pos;

inline void leddrv_update(void)
{
    
    uint8_t mask = 1;
    for(uint8_t i = 0; i < LEDDRV_STATES_CNT; i++)
    {
        for(uint8_t j = LEDDRV_CHANNELS; j > 0;)
        {
            j--;
            leddrv_states[i] <<= 1;
            if(leddrv_brightness[j] & mask)
                leddrv_states[i]++;

        }
        leddrv_states[i] &= LEDDRV_PORTMASK;
        mask <<= 1;
    }
}

void leddrv_set(uint8_t led, uint8_t brightness)
{
    leddrv_brightness[led] = brightness;

}

uint8_t leddrv_get(uint8_t led)
{
    return leddrv_brightness[led];

}

void leddrv_init()
{
    DDRA |= LEDDRV_PORTMASK;
    LEDDRV_PORT |= LEDDRV_PORTMASK;

    TCCR0A = (0<<COM0A1) | (0<<COM0A0) | (0<<COM0B1) | (0<<COM0B0) | (1<<WGM01) | (0<<WGM00); //CTC
    TCCR0B = (0<<FOC0A) | (0<<FOC0B) | (0<<WGM02) | (0<<CS02) | (1<<CS01) | (1<<CS00); //div 64
    


    //TCCR0 = 3; // div 64
    leddrv_pos = 0;


}


bool leddrv_poll()
{
    do
    {
        if((TIFR0 & (1<<OCF0A)) == 0)
        {
            continue;
        }

        TIFR0 |= 1<<OCF0A;

        OCR0A = leddrv_times[leddrv_pos];

        LEDDRV_PORT = leddrv_states[leddrv_pos];

        leddrv_pos++;
        if(leddrv_pos >= LEDDRV_STATES_CNT)
        {
            // this is not timing critical
            leddrv_update();
            leddrv_pos = 0;

        }

    } while(leddrv_pos < 4);

    if(leddrv_pos == 7) return true;
    return false;

}